package com.njcn.mapper.advanced;

import com.njcn.pojo.advanced.EventAssObj;
import com.njcn.pojo.advanced.EventsQueryCondition;
import com.njcn.pojo.advanced.SagEvent;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface WaveformMapper {
    List<SagEvent> queryEventsByGUID(@Param("eventsQueryCondition") EventsQueryCondition eventsQueryCondition, @Param("list") List<Integer> list);

    void addNormalization(List<SagEvent> list);

    List<SagEvent> queryDataByGuid(String indexEventAss);

    void deleteDataByGuid(String indexEventAss);

    void updateDataByGuid(List<EventAssObj> list);

    void updateWaveForm(List<String> guidIndex);
}
